/*
 * \usr\c\menu.c
 *
 *	Copyright (C) 1984 by Larry Campbell, 73 Concord St., Maynard Mass.
 *
 *	This software may be freely copied and disseminated for
 *	noncommercial purposes, if and only if this entire copyright
 *	statement and notice is included intact.  This software, and
 *	the information contained herein, may not be used for commercial
 *	purposes without my prior written permission.
 *
 *	This module provides a general-purpose "windowing" menu handler
 */

#include <stdio.h>
#include "video.h"
#include "rainbow.h"

/* This array contains 132 blanks (ASCII 040) */

char
    blanks[132] =
"                                                                  \
                                                                  ";

extern struct saved_video
    *pop_up_boxed_text ();

/*
 * Simple-minded menu handler.
 *
 *	menu_addr		Array of menu_type entries
 *	row, col		Position of upper left corner
 *	title			String used as caption
 *	flags			Various bits
 *
 */

int (*parse_menu ()) (menu_addr, row, col, title, flags)
struct menu_item menu_addr[];
int row, col, flags;
char *title;
{
#define PROMPT "Select, then press DO:"

int
    prompt_length = strlen (PROMPT),
    i,
    length,
    width,
    safety_width,
    (*handler_routine) (),
    selection,
    old_selection,
    selection_row,
    selection_col,
    help_present,
    do_flag,
    c;

char
    buf[132];

struct menu_item
    *item;

struct saved_screen_rectangle
    *saved_region,
    *help_box;

width = 0;
help_present = 0;
for (length = 0; (menu_addr[length].handler) != 0; length++)
    if ((i = strlen (menu_addr[length].name)) > width)
	width = i;
if ((i = strlen (title)) > width) width = i;
if (length > 9)
    abort ("Menu >9 items not supported");

safety_width = width + 6;		/* includes sides and numbers */
if ((prompt_length + 2) > safety_width)
    safety_width = prompt_length + 2;
saved_region = save_screen_rectangle (row, col, length + 5, safety_width);
paint_menu (menu_addr, row, col, title, flags, length, width);

/* Now handle menu selection.  If MENU_ACTIVE_MODE, do this repeatedly,
 * calling semantic routines until user chooses DONE entry. */

while (1)
    {
    putline (length + row + 4, col + 1, PROMPT);
    putattr_i (length + row + 4, col + 1, prompt_length, ATTR_BOLD);
    selection_row = row + length + 4;
    selection_col = col + prompt_length + 2;
    selection = 0;
    old_selection = 0;
    while (1)
        {
        do_flag = 0;
        cursor (selection_row, selection_col);	/* blink in useful place */
	buf[0] = ' ';
	if (selection) buf[1] = selection + '0'; else buf[1] = ' ';
	buf[2] = 0;
	putline (selection_row, selection_col - 1, buf);
        while ((c = kbdin ()) < 0);	/* poll keyboard until real char */
	if (help_present)	/* zap previous help box if necessary */
	    {
	    restore_screen_rectangle (help_box);
	    help_present = 0;
	    };
	c = map_keypad_numbers (c);	/* map keypad numbers to normal ones */
	if ((! (c & 0x100)) && ((c & 0xFF) == 015))
	    c = KEY_DO | 0x100;		/* make RETURN synonym for DO */
        if (c & 0x100)			/* function key? */
            {
            switch (c & 0xFF)
                {
                case KEY_DOWN_ARROW:
                    selection++; if (selection > length) selection = 1; break;
                case KEY_UP_ARROW:
                    selection--; if (selection < 1) selection = length; break;
		case KEY_KEYPAD_ENTER:
                case KEY_DO:
		    if (selection != 0)		/* if something chosen */
			{ do_flag = 1; break; }; /* go for it */
		    FEEP;			/* else feep and fall thru */
                case KEY_HELP:			/* to help */
                    help_box =
			pop_up_boxed_text (22, 2,
"Use arrow keys to move up and down, or type the number of the item you want");
		    help_present = 1;
                    break;
                default:
#ifdef DEBUG
		    sprintf (buf, "Invalid character 0x%3x", c);
		    boxed_text (10, 10, buf);
#endif
		    FEEP;
                };
            }
        else
            {			/* Got a real character, see if number */
            int i;
            i = (c & 0xFF) - '0';
            if ((i < 1) || (i > length))
		FEEP;
            else
                selection = i;
            };
        if (old_selection != selection)
            {
            putattr_i (row + 2 + old_selection, col + 1, width + 4, ATTR_NORMAL);
            putattr_i (row + 2 + selection, col + 1, width + 4, ATTR_REVERSE);
            };
        old_selection = selection;
        if (do_flag) break;
        };
    item = &menu_addr[selection - 1];

/* Erase prompt line */

    put_line (length + row + 4, col + 1, prompt_length + 2, blanks);
    putattr_i (length + row + 4, col + 1, prompt_length + 2, ATTR_NORMAL);

/* If not active mode, just return selection to caller */

    if (! (flags & MENU_ACTIVE_MODE))
	{
        if (flags & MENU_POST_CLEAR)
            menu_clear_region (row, col, length, width);
	restore_screen_rectangle (saved_region);
	return (item->handler);
	};

/* If active mode, call routine or return if user chose DONE */

    if (item->handler == DONE)
	{
	restore_screen_rectangle (saved_region);
	return (DONE);
	};
    handler_routine = item->handler;
    (*handler_routine) ();
    putattr_i (row + 2 + selection, col + 1, width + 4, ATTR_NORMAL);

/* If MENU_ONCE_ONLY, restore screen and return to caller */

    if (flags & MENU_ONCE_ONLY)
	{
	restore_screen_rectangle (saved_region);
	return (DONE);
	};
    };
}

paint_menu (menu_addr, row, col, title, flags, length, width)
struct menu_item menu_addr[];
int row, col, flags, length, width;
char *title;
{
char buf[132];

int i;

/* Clear screen, or menu region */

disable_cursor ();
if (flags & MENU_CLEAR_SCREEN)
    clear_screen ();
else					/* clear rectangle to be occupied */
    menu_clear_region (row, col, length, width);

/* Now either quickly or slowly (animated) draw menu outlines */

if (flags & MENU_ANIMATE)
    for (i = 1; i <= length + 3; i++)	/* animatedly draw rectangle */
	{
	put_line (row + i - 1, col + 1, width + 4, blanks);
	rectangle (row, col, i, width + 5, ATTR_NORMAL);
	}
else
    rectangle (row, col, length + 3, width + 5, ATTR_NORMAL);
horizontal_cut (row + 2, col, width + 6, ATTR_NORMAL);	/* title line */

/* Put title, reverse video, centered in title box at top of menu */

putattr_i (row + 1, col + 1, width + 4, ATTR_REVERSE);
putline (row + 1, col + ((width+4)/2) - (strlen (title) / 2) + 1, title);

/* Write the menu items and their numbers */

for (i = 0; (menu_addr[i].handler) != 0; i++)
    {
    sprintf (buf, " %d: %s", i + 1, menu_addr[i].name);
    putline (i + row + 3, col + 1, buf);
    };
enable_cursor ();
}

menu_clear_region (row, col, length, width)
{
int
    i;

disable_cursor ();
for (i = row; i <= row + length + 4; i++)	/* clear menu */
    {
    put_line (i, col, width + 5, blanks);
    putattr_i (i, col, width + 5, ATTR_NORMAL);
    };
enable_cursor ();
}

map_keypad_numbers (c)
int c;
{
if (c & 0x100)				/* if function key */
    switch (c & 0xFF)
	{
	case KEY_KEYPAD_0: c = '0'; break;
	case KEY_KEYPAD_1: c = '1'; break;
	case KEY_KEYPAD_2: c = '2'; break;
	case KEY_KEYPAD_3: c = '3'; break;
	case KEY_KEYPAD_4: c = '4'; break;
	case KEY_KEYPAD_5: c = '5'; break;
	case KEY_KEYPAD_6: c = '6'; break;
	case KEY_KEYPAD_7: c = '7'; break;
	case KEY_KEYPAD_8: c = '8'; break;
	case KEY_KEYPAD_9: c = '9'; break;
	case KEY_KEYPAD_ENTER: c = KEY_DO | 0x100; break;
	};
return (c);
}
